package main

import (
	"encoding/json"
	"errors"
	"flag"
	"fmt"
	"log"
	"os"
	"strconv"
	"time"

	"github.com/evanphx/json-patch/v5"
	"scm.dairydemon.net/filifa/mlblive/internal/statsapi"
)

func writeTest(v any, ts string) {
	b, err := json.Marshal(v)
	if err != nil {
		log.Println(err)
	}

	os.WriteFile("examples/data/data_"+ts+".json", b, 0644)
}

func getGamePk() string {
	teamId := flag.String("t", "", "team to get updates for")
	flag.Parse()
	if *teamId == "" {
		log.Fatal("need team ID")
	}

	id, ok := statsapi.TeamIds[*teamId]
	if !ok {
		log.Fatal("invalid team ID")
	}

	sched, err := statsapi.RequestSchedule("1", strconv.Itoa(id))
	if err != nil {
		log.Fatal(err)
	}

	var s statsapi.Schedule
	err = json.Unmarshal(sched, &s)
	if err != nil {
		log.Fatal(err)
	}

	gamePk := s.Dates[0].Games[0].GamePk.String()
	return gamePk
}

func extractPatch(diffPatch []byte) (jsonpatch.Patch, error) {
	var p jsonpatch.Patch
	var d []any
	err := json.Unmarshal(diffPatch, &d)
	if err != nil {
		return p, err
	}

	if len(d) == 0 {
		return p, errors.New("empty diffPatch")
	}

	diff := d[0].(map[string]any)["Diff"]
	patch, err := json.Marshal(diff)
	if err != nil {
		return p, err
	}

	p, err = jsonpatch.DecodePatch(patch)
	return p, err
}

func main() {
	gamePk := getGamePk()

	ws, err := statsapi.NewGamedayWebsocket(gamePk)
	if err != nil {
		log.Fatal(err)
	}
	defer ws.Close()

	ch := make(chan error)
	go ws.KeepAlive(10*time.Second, ch)

	feedResp, err := statsapi.RequestFeed(gamePk)
	if err != nil {
		log.Fatal(err)
	}

	var feed statsapi.Feed
	err = json.Unmarshal(feedResp, &feed)
	if err != nil {
		log.Fatal(err)
	}

	var game Game
	game.Update(&feed)

	ts := feed.MetaData.TimeStamp

	for {
		var p statsapi.Push
		err = ws.ReadJSON(&p)
		if err != nil {
			log.Fatal(err)
		}

		diffPatchResp, err := statsapi.RequestDiffPatch(gamePk, ts, p.UpdateId)
		patch, err := extractPatch(diffPatchResp)
		if err != nil {
			feedResp, err = statsapi.RequestFeed(gamePk)
			if err != nil {
				log.Fatal(err)
			}
		} else {
			feedResp, err = patch.Apply(feedResp)
			if err != nil {
				log.Fatal(err)
			}
		}

		err = json.Unmarshal(feedResp, &feed)
		if err != nil {
			log.Fatal(err)
		}

		ts = feed.MetaData.TimeStamp

		game.Update(&feed)
		fmt.Println(game)
	}
}
